/*!
        @file    $Id:: gradientFlow.h #$

        @brief

        @author  <Sinya Aoki> saoki@het.ph.tsukuba.ac.jp

        @date    $LastChangedDate:: 2014-10-20 19:33:57 #$

        @version $LastChangedRevision: 1164 $
*/

#ifndef GRADIENTFLOW_INCLUDED
#define GRADIENTFLOW_INCLUDED

#include "action.h"
#include "staples.h"

#include "bridgeIO.h"
using Bridge::vout;

//! GradientFlow construction.

/*!
    This is written by S. Aoki, based on staple.h
                               [01 July 2012 S.Aoki]
    (Coding history will be recovered from trac.)
    YAML is implemented.       [14 Nov 2012 Y.Namekawa]
 */

//- parameters class
class Parameters_GradientFlow : virtual public Parameters
{
 public:
  Parameters_GradientFlow();
};
//- end

class GradientFlow
{
 protected:
  Bridge::VerboseLevel m_vl;

 private:
  int    m_order_RK;
  double m_Estep;
  int    m_Nstep;
  int    m_Nprec;

  Action  *m_action;
  Field_G m_iP, m_iPP, m_iP1, m_iP2;

  int m_Ndim;
  int m_Nvol;

 public:
  GradientFlow(Action *action)
    : m_vl(CommonParameters::Vlevel()),
      m_Ndim(CommonParameters::Ndim()),
      m_Nvol(CommonParameters::Nvol())
  {
    m_iP.reset(m_Nvol, m_Ndim);
    m_iPP.reset(m_Nvol, m_Ndim);
    m_iP1.reset(m_Nvol, m_Ndim);
    m_iP2.reset(m_Nvol, m_Ndim);

    m_Nprec  = 0;
    m_Nstep  = 0;
    m_Estep  = 0.0;

    m_action = action;
  }

  void update_U(double estep, Field_G& iP, Field_G& U);

  void set_parameters(const Parameters& params);
  void set_parameters(const int order_RK,
                      const double Estep, const int Nstep, const int Nprec);

  void set_parameter_verboselevel(const Bridge::VerboseLevel vl) { m_vl = vl; }

  void gradientFlow_3rd(Field_G& U);
  void gradientFlow_4th(Field_G& U);

  double evolve(Field_G& U);
};
#endif
