/*!
        @file    $Id: test_HotStart_Eigenvalue.cpp #$

        @brief

        @author  Hideo Matsufuru <hideo.matsufuru@kek.jp> (matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate: 2013-01-22 13:51:53 #$

        @version $LastChangedRevision: 875 $
*/

#include "parameterManager_YAML.h"
#include "parameters_factory.h"

#include "eigen_QR_Cmplx.h"

#include "bridgeIO.h"
using Bridge::vout;

#include "gaugeConfig.h"

#include <valarray>

#ifdef USE_TEST
#include "test.h"
#endif

#ifdef USE_TESTMANAGER_AUTOREGISTER
#include "testManager.h"
#endif

//====================================================================
//! Test of hot start.

/*!
   [19 Nov 2013 S.Ueda]
 */

namespace Test_HotStart {
  //- test-private parameters
  namespace {
    const std::string filename_input  = "test_HotStart_Eigenvalue.yaml";
    const std::string filename_output = "stdout";

    class Parameters_Test_HotStart : public Parameters {
     public:
      Parameters_Test_HotStart()
      {
        //Register_string("gauge_config_type_input", "NULL");

        Register_string("verbose_level", "NULL");

        Register_double("expected_result", 0.0);
      }
    };
  }

  //- prototype declaration
  int eigenvalue(void);

#ifdef USE_TESTMANAGER_AUTOREGISTER
  namespace {
    static const bool is_registered = TestManager::RegisterTest(
      "HotStart.Eigenvalue",
      eigenvalue
      );
  }
#endif

  //====================================================================
  int eigenvalue(void)
  {
    int Nc    = CommonParameters::Nc();
    int Nvol  = CommonParameters::Nvol();
    int Ndim  = CommonParameters::Ndim();
    int NinG  = 2 * Nc * Nc;

    const double pi = 4*atan(1);

    Parameters *params_test      = new Parameters_Test_HotStart;

    Parameters *params_all = new Parameters;

    params_all->Register_Parameters("Test_HotStart", params_test);

    ParameterManager_YAML params_manager;
    params_manager.read_params(filename_input, params_all);

    const string str_vlevel      = params_test->get_string("verbose_level");
#ifdef USE_TEST
    const double expected_result = params_test->get_double("expected_result");
#endif

    Bridge::VerboseLevel vl = vout.set_verbose_level(str_vlevel);

    Field_G *u = new Field_G(Nvol, Ndim);
    GaugeConfig::set_hot(u);

    Mat_SU_N link(Nc);
    Mat_SU_N I(Nc);

    double av = 0;
    double va = 0;
    Eigen_QR_Cmplx eigen_qr(Nc);
    std::valarray<double> vec(2*Nc);
    for (int site = 0; site < Nvol; ++site) {
      for (int mu = 0; mu < Ndim; ++mu) {
        vec = eigen_qr.solve(u->ptr(0, site, mu));
        for (int icolor = 0; icolor < Nc; ++icolor) {
          double arg = atan2(vec[2*icolor+1], vec[2*icolor]);

          vout.detailed(vl, "%10.8f\n", arg);

          av += arg;
          va += arg*arg;
        }
      }
    }

    double av_all = Communicator::reduce_sum(av);
    double va_all = Communicator::reduce_sum(va);
    int Nval = CommonParameters::Lvol() * Ndim * Nc;

    av = av_all / Nval;
    va = va_all / Nval - av * av;

    va = sqrt(va);

    vout.general(vl, "\n");
    vout.general(vl, "Random SU(%d):\n", Nc);
    vout.general(vl, "  # of eigenvalue  = %10d\n", Nval);
    vout.general(vl, "  ave              = %23.16e\n", av);
    vout.general(vl, "  var              = %23.16e\n", va);
    vout.general(vl, "  var(expected)    = %23.16e\n", pi/sqrt(3));

    double result = av;


    // #####  tidy up  #####
    delete params_test;
    delete params_all;

    delete u;


#ifdef USE_TEST
    return Test::verify(expected_result, result);
#else
    return EXIT_SUCCESS;
#endif
  }
} // namespace Test_WilsonLoop
